<?php

namespace Magenest\Salesforce\Controller\Adminhtml\Sync;

use Magenest\Salesforce\Model\Connector;
use Magenest\Salesforce\Model\Sync;
use Magento\Backend\App\Action\Context;
use Magento\Config\Model\ResourceModel\Config as ResourceModelConfig;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface as ScopeConfigInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Serialize\Serializer\Serialize;

class Queue extends \Magento\Backend\App\Action
{
    /**
     * @var Sync\Contact
     */
    protected $_contact;

    /**
     * @var Sync\Campaign
     */
    protected $_campaign;

    /**
     * @var Sync\Account
     */
    protected $_account;

    /**
     * @var Sync\Lead
     */
    protected $_lead;

    /**
     * @var Sync\Order
     */
    protected $_order;

    /**
     * @var Sync\Product
     */
    protected $_product;

    /**
     * @var Sync\Opportunity
     */
    protected $_opportunity;

    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var ResourceModelConfig
     */
    protected $_resourceConfig;

    /**
     * @var TypeListInterface
     */
    protected $_cacheTypeList;

    /**
     * @var Serialize
     */
    protected $_serializer;

    /**
     * Queue constructor.
     *
     * @param Context $context
     * @param Sync\Contact $contact
     * @param Sync\Campaign $campaign
     * @param Sync\Account $account
     * @param Sync\Lead $lead
     * @param Sync\Order $order
     * @param Sync\Product $product
     * @param Sync\Opportunity $opportunity
     * @param ScopeConfigInterface $scopeConfig
     * @param ResourceModelConfig $resourceConfig
     * @param Serialize $serialize
     * @param TypeListInterface $cacheTypeList
     */
    public function __construct(
        Context $context,
        Sync\Contact $contact,
        Sync\Campaign $campaign,
        Sync\Account $account,
        Sync\Lead $lead,
        Sync\Order $order,
        Sync\Product $product,
        Sync\Opportunity $opportunity,
        ScopeConfigInterface $scopeConfig,
        ResourceModelConfig $resourceConfig,
        Serialize $serialize,
        TypeListInterface $cacheTypeList
    ) {
        parent::__construct($context);
        $this->_contact        = $contact;
        $this->_campaign       = $campaign;
        $this->_account        = $account;
        $this->_lead           = $lead;
        $this->_order          = $order;
        $this->_product        = $product;
        $this->_opportunity    = $opportunity;
        $this->_scopeConfig    = $scopeConfig;
        $this->_resourceConfig = $resourceConfig;
        $this->_serializer = $serialize;
        $this->_cacheTypeList  = $cacheTypeList;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $response = [];
        try {
            /** Turn off auto sync due to saving custom attributes */
            $autoSyncValues = $this->getAutoSyncValues();
            $this->setAutoSyncValues([]);
            /** refresh config cache */
            $this->_cacheTypeList->cleanType('config');

            /** Sync Queue */
            $response += $this->_account->syncAllQueue();

            $response += $this->_contact->syncAllQueue();

            $response += $this->_lead->syncAllQueue();

            $response += $this->_campaign->syncAllQueue();

            $response += $this->_product->syncAllQueue();

            $response += $this->_order->syncAllQueue();

            $response += $this->_opportunity->syncAllQueue();

            /** Reset auto sync settings */
            $this->setAutoSyncValues($autoSyncValues);
            /** refresh config cache */
            $this->_cacheTypeList->cleanType('config');

            $this->messageManager->addSuccessMessage(
                __('All items in queue are synced. Check out Reports for result.')
            );
        } catch (\Exception $e) {
            $response = $this->_serializer->serialize($response);
            $this->messageManager->addErrorMessage(
                __('Something happen during syncing process. Detail: ' . $e->getMessage() . '. Response Log: ' . $response)
            );
        }
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $resultRedirect->setUrl($this->_redirect->getRefererUrl());
        return $resultRedirect;
    }

    /**
     * @return array
     */
    protected function getAutoSyncValues()
    {
        return [
            Connector::XML_PATH_SALESFORCE_CONTACT_ENABLE  => $this->_scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_CONTACT_ENABLE),
            Connector::XML_PATH_SALESFORCE_LEAD_ENABLE     => $this->_scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_LEAD_ENABLE),
            Connector::XML_PATH_SALESFORCE_ACCOUNT_ENABLE  => $this->_scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_ACCOUNT_ENABLE),
            Connector::XML_PATH_SALESFORCE_ORDER_ENABLE    => $this->_scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_ORDER_ENABLE),
            Connector::XML_PATH_SALESFORCE_PRODUCT_ENABLE  => $this->_scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_PRODUCT_ENABLE),
            Connector::XML_PATH_SALESFORCE_CAMPAIGN_ENABLE => $this->_scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_CAMPAIGN_ENABLE),
        ];
    }

    /**
     * @param array $values
     */
    protected function setAutoSyncValues($values = [])
    {
        if (count($values) == 0) {
            $this->_resourceConfig->saveConfig(Connector::XML_PATH_SALESFORCE_CONTACT_ENABLE, 0, 'default', 0);
            $this->_resourceConfig->saveConfig(Connector::XML_PATH_SALESFORCE_LEAD_ENABLE, 0, 'default', 0);
            $this->_resourceConfig->saveConfig(Connector::XML_PATH_SALESFORCE_ACCOUNT_ENABLE, 0, 'default', 0);
            $this->_resourceConfig->saveConfig(Connector::XML_PATH_SALESFORCE_ORDER_ENABLE, 0, 'default', 0);
            $this->_resourceConfig->saveConfig(Connector::XML_PATH_SALESFORCE_PRODUCT_ENABLE, 0, 'default', 0);
            $this->_resourceConfig->saveConfig(Connector::XML_PATH_SALESFORCE_CAMPAIGN_ENABLE, 0, 'default', 0);
        } else {
            foreach ($values as $key => $value) {
                $this->_resourceConfig->saveConfig($key, $value, 'default', 0);
            }
        }
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Magenest_Salesforce::config_salesforce');
    }
}
