<?php

namespace Magenest\Salesforce\Helper;

use Magenest\Salesforce\Model\Connector;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Module\ModuleListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

/**
 * Class Data
 * @package Magenest\Salesforce\Helper
 */
class Data extends AbstractHelper
{
    const MODULE_NAME = 'Magenest_Salesforce';
    const CONFIGURATION_SALESFORCE_SECTION_PATH = 'adminhtml/system_config/edit/section/salesforcecrm';
    const CONFIGURATION_IS_CONNECTED = 'salesforcecrm/config/is_connected';
    const XML_PATH_INSTANCE_URL = 'salesforcecrm/config/instance_url';
    const XML_PATH_SALESFORCE_EMAIL = 'salesforcecrm/config/email';
    const XML_PATH_SALESFORCE_PASSWD = 'salesforcecrm/config/passwd';
    const XML_PATH_SALESFORCE_SECURITY_TOKEN = 'salesforcecrm/config/security_token';
    const XML_SYNC_CONFIGURATION_PATH = 'salesforcecrm/sync/';
    const IS_LINLKED_CONTACT_ACCOUNT = 'salesforcecrm/sync/linked_account';
    const ENABLE_PERSONAL_ACCOUNT = 'salesforcecrm/sync/personal_account';
    const ENABLE_DEBUG = 'salesforcecrm/sync/debug_mode';

    protected $enableDebug = null;

    /** @var \Magenest\Salesforce\Logger\Logger */
    protected $_logger;

    /** @var \Magento\Framework\Serialize\Serializer\Json */
    protected $_jsonFramework;

    /**
     * @var ModuleListInterface
     */
    protected $moduleList;

    /** @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory */
    protected $_orderColFactory;

    /** @var \Magento\Framework\UrlInterface */
    protected $_frontendUrl;

    /** @var WriterInterface */
    protected $_writer;

    /** @var ResourceConnection  */
    protected $_resourceConnection;

    /**
     * Data constructor.
     * @param Context $context
     * @param ModuleListInterface $moduleList
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderColFactory
     * @param \Magenest\Salesforce\Logger\Logger $logger
     * @param \Magento\Framework\Serialize\Serializer\Json $jsonFramework
     * @param WriterInterface $writer
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        Context $context,
        ModuleListInterface $moduleList,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderColFactory,
        \Magenest\Salesforce\Logger\Logger $logger,
        \Magento\Framework\Serialize\Serializer\Json $jsonFramework,
        WriterInterface $writer,
        ResourceConnection $resourceConnection
    ) {
        parent::__construct($context);
        $this->moduleList = $moduleList;
        $this->_orderColFactory = $orderColFactory;
        $this->_logger = $logger;
        $this->_jsonFramework = $jsonFramework;
        $this->_frontendUrl = $context->getUrlBuilder();
        $this->_writer = $writer;
        $this->_resourceConnection = $resourceConnection;
    }

    /**
     * @return array|null
     */
    public function getModule()
    {
        return $this->moduleList->getOne(self::MODULE_NAME);
    }

    /**
     * @return string
     */
    public function getModuleVersion()
    {
        return isset($this->getModule()['setup_version']) ? $this->getModule()['setup_version'] : '';
    }

    /**
     * @return bool
     */
    public function isSalesForceCRMConnected()
    {
        return $this->scopeConfig->isSetFlag(self::CONFIGURATION_IS_CONNECTED);
    }

    /**
     * @return mixed
     */
    public function getInstanceUrl()
    {
        return $this->scopeConfig->getValue(self::XML_PATH_INSTANCE_URL);
    }

    /**
     * @return mixed
     */
    public function getEmail()
    {
        return $this->scopeConfig->getValue(self::XML_PATH_SALESFORCE_EMAIL);
    }

    /**
     * @return mixed
     */
    public function getPassword()
    {
        return $this->scopeConfig->getValue(self::XML_PATH_SALESFORCE_PASSWD);
    }

    /**
     * @return mixed
     */
    public function getToken()
    {
        return $this->scopeConfig->getValue(self::XML_PATH_SALESFORCE_SECURITY_TOKEN);
    }

    /**
     * @param $pathXml
     * @return mixed
     */
    public function getConfigurationByPath($pathXml)
    {
        return $this->scopeConfig->getValue($pathXml);
    }

    /**
     * @param string $type
     * @return bool
     */
    public function getEnabledSyncConfig($type)
    {
        return (bool)$this->scopeConfig->getValue(self::XML_SYNC_CONFIGURATION_PATH . $type);
    }

    /**
     * @param string $type
     * @return int
     */
    public function getSyncConfigMode($type)
    {
        return $this->scopeConfig->getValue(self::XML_SYNC_CONFIGURATION_PATH . $type . '_mode');
    }

    /**
     * @param string $type
     * @return int
     */
    public function getSyncConfigTime($type)
    {
        return $this->scopeConfig->getValue(self::XML_SYNC_CONFIGURATION_PATH . $type . '_time');
    }

    /**
     * @return mixed
     */
    public function getConnectEnvironment()
    {
        return $this->scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_CONNECT_ENVIRONMENT);
    }

    /**
     * @param int $customerId
     *
     * @return \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    public function getCustomerOrderCollection($customerId)
    {
        return $this->_orderColFactory->create()->addFieldToFilter('customer_id', $customerId);
    }

    /**
     * @return bool
     */
    public function isLinkedContactWithAccount()
    {
        return $this->scopeConfig->isSetFlag(self::IS_LINLKED_CONTACT_ACCOUNT);
    }

    /**
     * @return bool
     */
    public function enablePersonalAccount()
    {
        return $this->scopeConfig->isSetFlag(self::ENABLE_PERSONAL_ACCOUNT);
    }

    /**
     * @return bool
     */
    public function enableDebug()
    {
        return $this->scopeConfig->isSetFlag(self::ENABLE_DEBUG);
    }

    /**
     * @param $message
     */
    public function saveLog($message)
    {
        if ($this->enableDebug()) {
            if (is_array($message)) {
                $this->_logger->info(var_export($message, true));
            } elseif (is_string($message)) {
                $this->_logger->info($message);
            } else {
                $this->_logger->info($this->_jsonFramework->serialize($message));
            }
        }
    }

    /**
     * @param $path
     * @return string
     */
    public function getFrontendUrlByPath($path)
    {
        return $this->_frontendUrl->getUrl($path);
    }

    /**
     * @param $path
     * @param $value
     */
    public function saveConfig($path, $value)
    {
        $this->_writer->save($path, $value, 'default', 0);
    }

    /**
     * @param $path
     * @param string $scope
     * @param int $scopeId
     * @return mixed|null
     */
    public function getCredentials($path, $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT, $scopeId = 0)
    {
        $value = null;
        $coreTable = $this->_resourceConnection->getTableName('core_config_data');
        $connection = $this->_resourceConnection->getConnection();
        $select = $connection->select()->from(
            $coreTable
        )->where(
            'path = ?',
            $path
        )->where(
            'scope = ?',
            $scope
        )->where(
            'scope_id = ?',
            $scopeId
        );
        $row = $connection->fetchRow($select);
        if ($row && isset($row['value'])) {
            $value = $row['value'];
        }
        return $value;
    }
}
