<?php

namespace Magenest\Salesforce\Model;

use Exception;
use Magenest\Salesforce\Helper\Constant;
use Magenest\Salesforce\Logger\Logger;
use Magenest\Salesforce\Model\ResourceModel\Queue as QueueResource;
use Magenest\Salesforce\Model\ResourceModel\Queue\CollectionFactory as QueueCollectionFactory;
use Magenest\Salesforce\Model\ResourceModel\Report\CollectionFactory as ReportCollectionFactory;
use Magento\Config\Model\ResourceModel\Config as ResourceModelConfig;
use Magento\Customer\Model\Customer;
use Magento\Framework\App\Config\ScopeConfigInterface as ScopeConfigInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\Serialize\Serializer\Serialize;
use Magento\Store\Model\ScopeInterface;

/**
 * Class Connector
 * @package Magenest\Salesforce\Model
 */
class Connector
{
    const SALES_FORCE_API_VERSION = '45.0';
    /**
     *#@+
     * Constants
     */
    const SALESFORCE_CONFIGURATION_SECTION        = 'adminhtml/system_config/edit/section/salesforcecrm';
    const XML_PATH_SALESFORCE_IS_CONNECTED        = 'salesforcecrm/config/is_connected';
    const XML_PATH_SALESFORCE_EMAIL               = 'salesforcecrm/config/email';
    const XML_PATH_SALESFORCE_PASSWD              = 'salesforcecrm/config/passwd';
    const XML_PATH_SALESFORCE_CLIENT_ID           = 'salesforcecrm/config/client_id';
    const XML_PATH_SALESFORCE_CLIENT_SECRET       = 'salesforcecrm/config/client_secret';
    const XML_PATH_SALESFORCE_SECURITY_TOKEN      = 'salesforcecrm/config/security_token';
    const XML_PATH_SALESFORCE_CONNECT_ENVIRONMENT = 'salesforcecrm/config/connect_environment';
    const XML_PATH_SALESFORCE_ACCESS_TOKEN        = 'salesforcecrm/config/access_token';
    const XML_PATH_SALESFORCE_REFRESH_TOKEN       = 'salesforcecrm/config/refresh_token';
    const XML_PATH_SALESFORCE_INSTANCE_URL        = 'salesforcecrm/config/instance_url';
    const XML_PATH_SALESFORCE_CONTACT_ENABLE      = 'salesforcecrm/sync/contact';
    const XML_PATH_SALESFORCE_ACCOUNT_ENABLE      = 'salesforcecrm/sync/account';
    const XML_PATH_SALESFORCE_LEAD_ENABLE         = 'salesforcecrm/sync/lead';
    const XML_PATH_SALESFORCE_CAMPAIGN_ENABLE     = 'salesforcecrm/sync/campaign';
    const XML_PATH_SALESFORCE_ORDER_ENABLE        = 'salesforcecrm/sync/order';
    const XML_PATH_SALESFORCE_PRODUCT_ENABLE      = 'salesforcecrm/sync/product';
    const XML_STANDARD_PRICE_BOOK_ID_PATH         = 'salesforcecrm/pricebook/standard_pricebook_id';

    const IGNORE_FIELD_TYPES = ['picklist', 'reference', 'calculated', 'multipicklist'];// array('date', 'datetime', 'address')

    protected $_reportCollectionFactory;

    /** @var \Magenest\Salesforce\Helper\Data  */
    protected $_helperData;

    /**
     * @var string serialize string
     */
    protected $fields;

    /**
     * @var string serialize string
     */
    protected $fieldsType = null;

    /**
     * Core store config
     *
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     *
     * @var \Magento\Config\Model\ResourceModel\Config
     */
    protected $_resourceConfig;

    /**
     * @var \Magenest\Salesforce\Model\ReportFactory
     */
    protected $_reportFactory;

    /**
     * @var string
     */
    protected $_type;

    /**
     * @var string
     */
    protected $_table;

    /** @var QueueCollectionFactory  */
    protected $_queueCollection;

    /** @var QueueResource  */
    protected $_queueResource;

    /** @var Serialize  */
    protected $_serializer;

    /** @var RequestLogFactory  */
    protected $_requestLogFactory;

    protected $credential = null;

    protected $requestsSent = 0;

    /** @var Json  */
    protected $_json;

    /** @var Logger  */
    protected $_logger;

    protected $_standardPriceBookId = null;

    /**
     * Connector constructor.
     * @param ReportCollectionFactory $reportCollectionFactory
     * @param \Magenest\Salesforce\Helper\Data $helperData
     * @param Json $json
     * @param Serialize $serialize
     * @param ScopeConfigInterface $scopeConfig
     * @param ResourceModelConfig $resourceConfig
     * @param ReportFactory $reportFactory
     * @param QueueCollectionFactory $queueCollectionFactory
     * @param QueueResource $queueResource
     * @param RequestLogFactory $requestLogFactory
     * @param Logger $logger
     */
    public function __construct(
        ReportCollectionFactory $reportCollectionFactory,
        \Magenest\Salesforce\Helper\Data $helperData,
        Json $json,
        Serialize $serialize,
        ScopeConfigInterface $scopeConfig,
        ResourceModelConfig $resourceConfig,
        ReportFactory $reportFactory,
        QueueCollectionFactory $queueCollectionFactory,
        QueueResource $queueResource,
        RequestLogFactory $requestLogFactory,
        Logger $logger
    ) {
        $this->_reportCollectionFactory = $reportCollectionFactory;
        $this->_helperData = $helperData;
        $this->_json = $json;
        $this->_serializer = $serialize;
        $this->_scopeConfig       = $scopeConfig;
        $this->_resourceConfig    = $resourceConfig;
        $this->_reportFactory     = $reportFactory;
        $this->_queueCollection = $queueCollectionFactory;
        $this->_queueResource = $queueResource;
        $this->_requestLogFactory = $requestLogFactory;
        $this->_logger = $logger;
    }

    /**
     * @param $magentoId
     * @return \Magento\Framework\DataObject
     */
    public function checkLogReportByMagentoId($magentoId)
    {
        return $this->_reportCollectionFactory->create()
            ->addFieldToFilter('salesforce_table', $this->_type)
            ->addFieldToFilter('magento_id', $magentoId)
            ->getLastItem();
    }

    /**
     * @param $data
     */
    public function setCredential($data)
    {
        $this->credential = $data;
    }

    /**
     * @param array $data
     * @param false $update
     * @return array|bool|float|int|mixed|string|null
     * @throws \Exception
     */
    public function getAccessToken($data = [], $update = false)
    {
        try {
            if ((!empty($data) && $update)) {
                $username            = $data['username'];
                $password            = $data['password'];
                $client_id           = $data['client_id'];
                $client_secret       = $data['client_secret'];
                $security_token      = $data['security_token'];
                $connect_environment = $data['connect_environment'];
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_EMAIL, $data['username'], 'default', 0);
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_PASSWD, $data['password'], 'default', 0);
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_CLIENT_ID, $data['client_id'], 'default', 0);
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_CLIENT_SECRET, $data['client_secret'], 'default', 0);
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_SECURITY_TOKEN, $data['security_token'], 'default', 0);
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_CONNECT_ENVIRONMENT, $data['connect_environment'], 'default', 0);
            } elseif (!is_null($this->credential)) {
                $data                = $this->credential;
                $username            = $data['username'];
                $password            = $data['password'];
                $client_id           = $data['client_id'];
                $client_secret       = $data['client_secret'];
                $security_token      = $data['security_token'];
                $connect_environment = $data['connect_environment'];
            } else {
                $username            = $this->_scopeConfig->getValue(self::XML_PATH_SALESFORCE_EMAIL);
                $password            = $this->_scopeConfig->getValue(self::XML_PATH_SALESFORCE_PASSWD);
                $client_id           = $this->_scopeConfig->getValue(self::XML_PATH_SALESFORCE_CLIENT_ID);
                $client_secret       = $this->_scopeConfig->getValue(self::XML_PATH_SALESFORCE_CLIENT_SECRET);
                $security_token      = $this->_scopeConfig->getValue(self::XML_PATH_SALESFORCE_SECURITY_TOKEN);
                $connect_environment = $this->_scopeConfig->getValue(self::XML_PATH_SALESFORCE_CONNECT_ENVIRONMENT);
            }

            if (!$username || !$password || !$client_id || !$client_secret || !$security_token) {
                throw new LocalizedException(__('Please fill in all required fields!'));
            }

            if ($connect_environment === 1 || $connect_environment === "1") {
                $base_url = 'https://login.salesforce.com/';
            } else {
                $base_url = 'https://test.salesforce.com/';
            }
            $url      = $base_url . 'services/oauth2/token';
            $params   = [
                'grant_type'    => 'password',
                'client_id'     => $client_id,
                'client_secret' => $client_secret,
                'username'      => $username,
                'password'      => $password . $security_token
            ];
            $response = $this->makeRequest(\Zend_Http_Client::POST, $url, [], $params);
            $response = $this->_json->unserialize($response);
            if (isset($response['access_token']) && isset($response['instance_url'])) {
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_INSTANCE_URL, $response['instance_url'], 'default', 0);
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_ACCESS_TOKEN, $response['access_token'], 'default', 0);
                $this->_resourceConfig->saveConfig(self::XML_PATH_SALESFORCE_IS_CONNECTED, 1, 'default', 0);
                unset($response['id']);
                unset($response['token_type']);
                unset($response['signature']);
                unset($response['issued_at']);

                return $response;
            } else {
                throw new \Exception($response['error_description']);
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    /**
     * @param $method
     * @param $path
     * @param null $parameter
     * @param false $useFreshCredential
     * @return array|bool|float|int|mixed|string|null
     * @throws \Zend_Http_Client_Exception
     * @throws Exception
     */
    public function sendRequest($method, $path, $parameter = null, $useFreshCredential = false)
    {
        $instance_url = $this->_helperData->getCredentials(self::XML_PATH_SALESFORCE_INSTANCE_URL);
        $access_token = $this->_helperData->getCredentials(self::XML_PATH_SALESFORCE_ACCESS_TOKEN);
        $response = [];
        try {
            if ($useFreshCredential) {
                //Request an Updated Access Token
                $login = $this->updateAccessToken();
                $instance_url = $login['instance_url'] ?? "#";
                $access_token = $login['access_token'] ?? null;
                $this->_helperData->saveConfig(self::XML_PATH_SALESFORCE_INSTANCE_URL, $instance_url);
                $this->_helperData->saveConfig(self::XML_PATH_SALESFORCE_ACCESS_TOKEN, $access_token);
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }

        $headers = ["Content-Type" => "application/json"];
        $url      = $instance_url . $path;
        if (isset($access_token)) {
            $headers = array_merge($headers, ["Authorization" => "Bearer " . $access_token]);
            $response = $this->makeRequest($method, $url, $headers, $parameter);
        } else {
            $response = $this->sendRequest($method, $path, $parameter, true);
        }
        if (!empty($response) && !is_array($response)) {
            $response = $this->_json->unserialize($response);
        }
        if (isset($response[0]['errorCode']) && $response[0]['errorCode'] == 'INVALID_SESSION_ID') {
            $response = $this->sendRequest($method, $path, $parameter, true);
        }

        return $response;
    }

    /**
     * @return array|bool|float|int|mixed|string|null
     * @throws Exception
     */
    protected function updateAccessToken()
    {
        try {
            $connect_environment = $this->_scopeConfig->getValue(Constant::XML_PATH_SALESFORCE_CONNECT_ENVIRONMENT);
            $salesforceInstanceURL = Constant::INSTANCE_SF_SANDBOX_MODE;
            if ($connect_environment === 1 || $connect_environment === "1") {
                $salesforceInstanceURL = Constant::INSTANCE_SF_PRODUCTION_MODE;
            }
            $url = $salesforceInstanceURL . "services/oauth2/token";
            $clientId = $this->_scopeConfig->getValue(Constant::XML_PATH_SALESFORCE_CLIENT_ID);
            $clientSecret = $this->_helperData->getConfigurationByPath(Connector::XML_PATH_SALESFORCE_CLIENT_SECRET);
            $refreshToken = $this->_helperData->getConfigurationByPath(Connector::XML_PATH_SALESFORCE_REFRESH_TOKEN);
            $params = [
                'grant_type' => "refresh_token",
                'client_id' => $clientId,
                'client_secret' => $clientSecret,
                'refresh_token' => $refreshToken
            ];
            $response = $this->makeRequest(\Zend_Http_Client::POST, $url, [], $params);
            return $this->_json->unserialize($response);
        } catch (\Exception $exception) {
            throw new \Exception($exception->getMessage());
        }
    }

    /**
     * @param $table
     * @param $parameter
     * @param null $mid
     * @return bool|mixed
     * @throws LocalizedException
     * @throws \Zend_Http_Client_Exception
     * @throws Exception
     */
    public function createRecords($table, $parameter, $mid = null)
    {
        try {
            $this->_helperData->saveLog("Create record with " . $this->_json->serialize($parameter));
            $path     = "/services/data/v" . self::SALES_FORCE_API_VERSION . "/sobjects/" . $table . "/";
            $response = $this->sendRequest(\Zend_Http_Client::POST, $path, $parameter);
            if (isset($response["id"])) {
                $id = $response["id"];
                $this->saveReport($id, 'create', $table, 1, null, $mid, $response);
                return $id;
            } elseif (isset($response[0]['errorCode'])) {
                $message = $response[0]['errorCode'] . ": " . $response[0]['message'];
                $this->saveReport(null, null, $this->_type, 2, $message, $mid, $response);
                throw new \Exception($message);
            }
        } catch (\Exception $exception) {
            throw new \Exception($exception->getMessage());
        }
    }

    /**
     * Delete a record in salesforce
     *
     * @param $table
     * @param $id
     * @param null $mid
     * @throws LocalizedException
     * @throws \Zend_Http_Client_Exception
     */
    public function deleteRecords($table, $id, $mid = null)
    {
        $path = "/services/data/v" . self::SALES_FORCE_API_VERSION . "/sobjects/" . $table . "/" . $id;
        $this->sendRequest(\Zend_Http_Client::DELETE, $path);
        $this->saveReport($id, 'delete', $table, 1, null, $mid);
    }

    /**
     * @param $table
     * @param $id
     * @param $parameter
     * @param null $mid
     *
     * @return mixed
     * @throws LocalizedException
     * @throws \Zend_Http_Client_Exception
     */
    public function updateRecords($table, $id, $parameter, $mid = null)
    {
        $this->_helperData->saveLog("Update record with " . $this->_json->serialize($parameter));
        $path = "/services/data/v" . self::SALES_FORCE_API_VERSION . "/sobjects/" . $table . "/" . $id;
        $response = $this->sendRequest(\Zend_Http_Client::PATCH, $path, $parameter);
        $this->saveReport($id, 'update', $table, 1, null, $mid, $response);
        return $id;
    }

    /**
     * @param $table
     * @param $field
     * @param $value
     * @param string $return
     * @return false|mixed|string
     * @throws \Zend_Http_Client_Exception
     */
    public function searchRecords($table, $field, $value, $return = '')
    {
        if ($table == 'PricebookEntry' && $return == 'array') {
            $query = "SELECT Id, Pricebook2Id, UnitPrice FROM $table WHERE $field = '$value' ";
            $query .= ' ORDER BY Id ';
        } else {
            $query = "SELECT Id FROM $table WHERE $field = '$value' ";
            $query .= 'LIMIT 1';
        }

        $path  = '/services/data/v' . self::SALES_FORCE_API_VERSION . '/query?q=' . urlencode($query);

        $response = $this->sendRequest(\Zend_Http_Client::GET, $path);
        if (isset($response['totalSize']) && $response['totalSize'] >= 1) {
            if ($table == 'PricebookEntry' && $return == 'array') {
                return $response['records'];
            } else {
                return $response['records']['0']['Id'];
            }
        }
        return false;
    }

    /**
     * @param $product2Id
     * @param $priceBook2Id
     * @return false|mixed|string
     * @throws \Zend_Http_Client_Exception
     */
    public function searchProductIntoPricebook2($product2Id, $priceBook2Id)
    {
        $query = "SELECT Id FROM 'PricebookEntry' WHERE 'Product2Id' = '$product2Id' AND 'Pricebook2Id' = '$priceBook2Id'  ORDER BY Id LIMIT 1";
        $path  = '/services/data/v' . self::SALES_FORCE_API_VERSION . '/query?q=' . urlencode($query);
        $response = $this->sendRequest(\Zend_Http_Client::GET, $path);
        if (isset($response['totalSize']) && $response['totalSize'] == 1) {
            return $response['records']['0']['Id'];
        }
        return false;
    }

    /**
     * @param $table
     * @return string
     * @throws LocalizedException
     * @throws \Zend_Http_Client_Exception
     */
    public function getFields($table)
    {
        if ($this->fields === null) {
            $path             = '/services/data/v' . self::SALES_FORCE_API_VERSION . '/sobjects/' . $table . '/describe/';
            $response         = $this->sendRequest(\Zend_Http_Client::GET, $path);
            $data             = [];
            $type             = [];
            $ignoreFieldTypes = $this->getIgnoreFieldTypes();
            if (isset($response['fields'])) {
                foreach ($response['fields'] as $index => $field) {
                    if ($field['createable'] && $field['updateable'] && !in_array($field['type'], $ignoreFieldTypes)) {
                        $label       = $field['label'] . " (" . $field['type'] . ")";
                        $name        = $field['name'];
                        $data[$name] = $label;
                        $type[$name] = $field['type'];
                    }
                }
            }
            $this->fields = $this->_serializer->serialize($data);
            $this->fieldsType = $this->_serializer->serialize($type);
        }
        $results = $this->_json->serialize([
            'fields' => $this->fields,
            'types' => $this->fieldsType
        ]);
        return $results;
    }

    /**
     * @return array
     */
    public function getIgnoreFieldTypes()
    {
        return self::IGNORE_FIELD_TYPES;
    }

    /**
     * @param $id
     * @param $action
     * @param $table
     * @param int $status
     * @param null $message
     * @param null $mid
     * @param null $response
     *
     * @throws LocalizedException
     */
    public function saveReport($id, $action, $table, $status = 1, $message = null, $mid = null, $response = null)
    {
        $model = $this->_reportFactory->create();
        $model->saveReport($id, $action, $table, $status, $message, $mid, $response);

        return;
    }

    /**
     * @param $action
     * @param $table
     * @param $response
     * @param $magentoIds
     *
     * @throws LocalizedException
     */
    public function saveReports($action, $table, $response, $magentoIds)
    {
        if (is_array($response)) {
            $total   = count($response);
            $reports = [];
            for ($i = 0; $i < $total; $i++) {
                $recordId  = null;
                $status    = 2;
                $message   = null;
                $magentoId = null;
                if (isset($response[$i]['success']) && $response[$i]['success']) {
                    $recordId  = $response[$i]['id'];
                    $status    = 1;
                    $magentoId = isset($magentoIds[$i]['mid']) ? $magentoIds[$i]['mid'] : null;
                } elseif (isset($response[$i]['errors'][0])) {
                    $message = 'ERROR ';
                    foreach ($response[$i]['errors'] as $error) {
                        $message .= $error['message'] . ';';
                    }
                } else {
                    $message = $this->_serializer->serialize($response);
                }
                $params    = [
                    'record_id'        => $recordId,
                    'action'           => $action,
                    'salesforce_table' => $table,
                    'status'           => $status,
                    'msg'              => $message,
                    'magento_id'       => $magentoId,
                    'response' => isset($response[$i]) ? $this->_json->serialize($response[$i]) : $this->_json->serialize($response)
                ];
                $params    += $this->_reportFactory->create()->getInfoReport();
                $reports[] = $params;
            }
            $this->_reportFactory->create()->saveReports($reports);
        }
    }

    /**
     * @param $method
     * @param $url
     * @param array $headers
     * @param array $params
     * @return string|null
     * @throws \Zend_Http_Client_Exception
     */
    public function makeRequest($method, $url, $headers = [], $params = [])
    {
        $client = new \Zend_Http_Client($url);
        $client->setHeaders($headers);
        if ($method != \Zend_Http_Client::GET) {
            $client->setParameterPost($params);
            if (isset($headers['Content-Type'])) {
                $client->setEncType($headers['Content-Type']);
                $params = $this->_json->serialize($params);
                $client->setRawData($params);
            }
        }
        $response = $client->request($method)->getBody();
        $this->_requestLogFactory->create()->addRequest(RequestLog::REST_REQUEST_TYPE);
        return $response;
    }

    /**
     * @return array
     * @throws Exception
     */
    public function syncAllQueue()
    {
        $type = $this->_type;
        if ($type == 'Product2') {
            $type = 'Products';
        }
        $queueCollection = $this->_queueCollection->create()
            ->addFieldToFilter('type', rtrim($type, 's'));

        $lastId    = (int)$queueCollection->getLastItem()->getId();
        $count     = 0;
        $response  = [];
        $maxRecord = 5000;
        /** @var \Magenest\Salesforce\Model\Queue $queue */
        foreach ($queueCollection as $queue) {
            $entityId = $queue->getEntityId();
            $this->addRecord($entityId);
            $count++;
            if ($count >= $maxRecord || $queue->getId() == $lastId) {
                $response += $this->syncQueue();
                $count    = 0;
            }
        }
        $count              = 0;
        $queueCollectionArr = [];
        foreach ($queueCollection as $queue) {
            $queueCollectionArr[] = $queue->getId();
            $count++;
            if ($count >= $maxRecord || $queue->getId() == $lastId) {
                $this->_queueResource->deleteMultiQueues($queueCollectionArr);
                $count              = 0;
                $queueCollectionArr = [];
            }
        }
        return $response;
    }

    /**
     * @return mixed|string
     * @throws \Exception
     */
    public function getRecordTypeId()
    {
        try {
            $query = "SELECT Id FROM RecordType WHERE IsPersonType = TRUE LIMIT 1";
            $path     = "/services/data/v" . self::SALES_FORCE_API_VERSION . "/query?q=" . urlencode($query);
            $response = $this->sendRequest(\Zend_Http_Client::GET, $path);
            if (isset($response['totalSize']) && $response['totalSize'] == 1) {
                $id = $response['records']['0']['Id'];
                return $id;
            }
        } catch (\Exception $exception) {
            $this->_logger->critical("Get RecordTypeId errors: " . $exception->getMessage());
            throw new \Exception($exception->getMessage());
        }
    }

    /**
     * @param $email
     * @return array
     * @throws \Zend_Http_Client_Exception
     */
    public function getCustomerByEmail($email, $table = 'Account')
    {
        $sfId = false;
        $query = "SELECT Id FROM $table WHERE Name LIKE '" . $email . "' LIMIT 1";
        $path  = '/services/data/v' . self::SALES_FORCE_API_VERSION . '/query?q=' . urlencode($query);
        $response = $this->sendRequest(\Zend_Http_Client::GET, $path);
        if (isset($response['totalSize']) && $response['totalSize'] >= 1 && isset($response['records'])) {
            $records = reset($response['records']);
            $sfId = $records['Id'];
        }
        return $sfId;
    }

    /**
     * @param Customer $customer
     * @param string $type
     * @param string $field
     * @return array|false|mixed|string|null
     * @throws \Zend_Http_Client_Exception
     */
    protected function checkSalesforceCustomerStatus(Customer $customer, $type = "Account", $field = "Name")
    {
        $customerInReport = $this->checkLogReportByMagentoId($customer->getId());
        if ($recordId = $customerInReport->getData('record_id')) {
            return $recordId;
        }
        return $this->searchRecords($type, $field, $customer->getEmail());
    }

    /**
     * @return bool|mixed|string
     * @throws LocalizedException
     * @throws \Zend_Http_Client_Exception
     */
    public function _getStandardPriceBookId()
    {
        if (!$this->_standardPriceBookId) {
            $pricebook2Id = $this->_scopeConfig->getValue(self::XML_STANDARD_PRICE_BOOK_ID_PATH);
            if (empty($pricebook2Id)) {
                $pricebook2Id = $this->searchRecords('Pricebook2', 'Name', 'Standard Price Book');
                $this->_resourceConfig->saveConfig(self::XML_STANDARD_PRICE_BOOK_ID_PATH, $pricebook2Id, 'default', 0);
            }
            $this->_standardPriceBookId = $pricebook2Id;
        }
        return $this->_standardPriceBookId;
    }

    /**
     * @param $product2Id
     * @param $pricebook2Id
     * @return false|mixed|string
     * @throws \Zend_Http_Client_Exception
     */
    public function getPricebookEntryId($product2Id, $pricebook2Id)
    {
        $pricebookEntryId = false;
        $listPricebookEntry = $this->searchRecords('PricebookEntry', 'Product2Id', $product2Id, 'array');
        if (is_array($listPricebookEntry) && !empty($listPricebookEntry)) {
            foreach ($listPricebookEntry as $priceBookEntry) {
                if (isset($priceBookEntry['Pricebook2Id']) && $priceBookEntry['Pricebook2Id'] == $pricebook2Id) {
                    $pricebookEntryId = $priceBookEntry['Id'];
                    break;
                }
            }
        }
        return $pricebookEntryId;
    }

    public function addRecord($entityId)
    {
    }

    public function syncQueue()
    {
        return null;
    }
}
