<?php

namespace Magenest\Salesforce\Model\Sync;

use Magenest\Salesforce\Logger\Logger;
use Magenest\Salesforce\Model\Connector;
use Magenest\Salesforce\Model\Data;
use Magenest\Salesforce\Model\ReportFactory as ReportFactory;
use Magenest\Salesforce\Model\RequestLogFactory;
use Magenest\Salesforce\Model\ResourceModel\Queue as QueueResource;
use Magenest\Salesforce\Model\ResourceModel\Queue\CollectionFactory as QueueCollectionFactory;
use Magenest\Salesforce\Model\ResourceModel\Report\CollectionFactory as ReportCollectionFactory;
use Magento\Config\Model\ResourceModel\Config as ResourceModelConfig;
use Magento\Customer\Model\Customer;
use Magento\Customer\Model\CustomerFactory;
use Magento\Framework\App\Config\ScopeConfigInterface as ScopeConfigInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\Serialize\Serializer\Serialize;

/**
 * Class Lead
 * @package Magenest\Salesforce\Model\Sync
 */
class Lead extends Connector
{
    /** @var CustomerFactory  */
    protected $_customerFactory;

    /** @var Job  */
    protected $_job;

    /** @var \Magenest\Salesforce\Helper\Data  */
    protected $helper;

    /** @var DataGetter  */
    protected $dataGetter;

    /** @var Data  */
    protected $_data;

    protected $existedLeads = null;

    protected $createLeadIds = null;

    protected $updateLeadIds = null;

    /**
     * Lead constructor.
     * @param Data $data
     * @param CustomerFactory $customerFactory
     * @param Job $job
     * @param DataGetter $dataGetter
     * @param ReportCollectionFactory $reportCollectionFactory
     * @param \Magenest\Salesforce\Helper\Data $helperData
     * @param Json $json
     * @param Serialize $serialize
     * @param ScopeConfigInterface $scopeConfig
     * @param ResourceModelConfig $resourceConfig
     * @param ReportFactory $reportFactory
     * @param QueueCollectionFactory $queueCollectionFactory
     * @param QueueResource $queueResource
     * @param RequestLogFactory $requestLogFactory
     * @param Logger $logger
     */
    public function __construct(
        Data $data,
        CustomerFactory $customerFactory,
        Job $job,
        DataGetter $dataGetter,
        ReportCollectionFactory $reportCollectionFactory,
        \Magenest\Salesforce\Helper\Data $helperData,
        Json $json,
        Serialize $serialize,
        ScopeConfigInterface $scopeConfig,
        ResourceModelConfig $resourceConfig,
        ReportFactory $reportFactory,
        QueueCollectionFactory $queueCollectionFactory,
        QueueResource $queueResource,
        RequestLogFactory $requestLogFactory,
        Logger $logger
    ) {
        $this->_data            = $data;
        $this->_customerFactory = $customerFactory;
        $this->_job             = $job;
        $this->dataGetter       = $dataGetter;
        $this->_type            = 'Lead';
        $this->_table           = 'customer';
        parent::__construct($reportCollectionFactory, $helperData, $json, $serialize, $scopeConfig, $resourceConfig, $reportFactory, $queueCollectionFactory, $queueResource, $requestLogFactory, $logger);
    }

    /**
     * @param $id
     * @param false $update
     * @return mixed|bool|string
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Zend_Http_Client_Exception
     */
    public function sync($id, $update = false)
    {
        /** @var Customer $model */
        $model = $this->_customerFactory->create()->load($id);
        $salesforceId = false;
        if (intval($this->getCustomerOrderCollection($model)->getSize()) == 0) {
            $email     = $model->getEmail();
            $firstname = $model->getFirstname();
            $lastname  = $model->getLastname();

            $salesforceId = $this->searchRecords($this->_type, 'Email', $email);
            if (!$salesforceId || ($update && $salesforceId)) {
                $params = $this->_data->getCustomer($model, $this->_type);
                $params += [
                    'FirstName' => $firstname,
                    'LastName'  => $lastname,
                    'Email'     => $email,
                ];
                if (empty($params['Company'])) {
                    $params['Company'] = 'N/A';
                }
                $this->_helperData->saveLog("Lead params mapping: " . $this->_json->serialize($params));
                if ($update && $salesforceId) {
                    $this->updateRecords($this->_type, $salesforceId, $params, $model->getId());
                } else {
                    $salesforceId = $this->createRecords($this->_type, $params, $model->getId());
                }
            }
        } else {
            $this->_helperData->saveLog("Customer has order");
        }
        return $salesforceId;
    }

    /**
     * Delete Record
     *
     * @param string $email
     * @throws \Exception
     */
    public function delete($email)
    {
        $leadId = $this->searchRecords('Lead', 'Email', $email);
        if ($leadId) {
            $this->deleteRecords('Lead', $leadId);
        }
    }

    /**
     * Sync by email
     *
     * @param string $email
     * @throws \Exception
     */
    public function syncByEmail($email)
    {
        $leadId = $this->searchRecords('Lead', 'Email', $email);
        if (!$leadId) {
            $params = [
                'Email'    => $email,
                'LastName' => 'Guest',
                'Company'  => 'N/A',
            ];
            $this->createRecords($this->_type, $params);
        }
    }

    /**
     * Sync All Customer on Magento to Salesforce
     */
    public function syncAllLead()
    {
        try {
            $customers      = $this->_customerFactory->create()->getCollection();
            $lastCustomerId = $customers->getLastItem()->getId();
            $count          = 0;
            $response       = [];
            /** @var \Magento\Customer\Model\Customer $customer */
            foreach ($customers as $customer) {
                $this->addRecord($customer->getId());
                $count++;
                if ($count >= 10000 || $customer->getId() == $lastCustomerId) {
                    $response += $this->syncQueue();
                }
            }
            return $response;
        } catch (\Exception $e) {
            $this->_logger->critical($e->getMessage());
        }
        return null;
    }

    /**
     * @return array|mixed|string|null
     */
    public function syncQueue()
    {
        $createResponse = $this->createLeads();
        $updateResponse = $this->updateLeads();
        $response       = $createResponse + $updateResponse;
        $this->unsetCreateQueue();
        $this->unsetUpdateQueue();
        return $response;
    }

    /**
     * Send request to create leads
     */
    protected function createLeads()
    {
        $response = [];
        if (!is_null($this->createLeadIds)) {
            $response = $this->sendLeadsRequest($this->createLeadIds, 'insert');
        }
        return $response;
    }

    /**
     * Send request to update leads
     */
    protected function updateLeads()
    {
        $response = [];
        if (!is_null($this->updateLeadIds)) {
            $response = $this->sendLeadsRequest($this->updateLeadIds, 'update');
        }
        return $response;
    }

    /**
     * @param $customerId
     * @throws \Exception
     */
    public function addRecord($customerId)
    {
        $customer = $this->_customerFactory->create()->load($customerId);
        $id = $this->checkSalesforceCustomerStatus($customer, "Lead", "Email");
        if (!$id) {
            if ($this->checkSalesforceCustomerStatus($customer, "Contact", "Email")) {
                $errorMessage = "This Lead cannot be synced,\n
                because a Contact with the same email address already existed.";
                $this->saveReport(null, "insert", $this->_type, 2, $errorMessage, $customerId);
            } else {
                $this->addToCreateQueue($customer);
            }
        } else {
            $this->addToUpdateQueue($customer, $id);
        }
    }

    /**
     * @param \Magento\Customer\Model\Customer $customer
     */
    protected function addToCreateQueue($customer)
    {
        $this->createLeadIds[] = [
            'mObj' => $customer,
            'mid'  => $customer->getId()
        ];
    }

    /**
     * @param \Magento\Customer\Model\Customer $customer
     * @param $salesforceId
     */
    protected function addToUpdateQueue($customer, $salesforceId)
    {
        $this->updateLeadIds[] = [
            'mObj' => $customer,
            'mid'  => $customer->getId(),
            'sid'  => $salesforceId
        ];
    }

    protected function unsetCreateQueue()
    {
        $this->createLeadIds = null;
    }

    protected function unsetUpdateQueue()
    {
        $this->updateLeadIds = null;
    }

    /**
     * @param array $leadIds
     * @param string $operation
     * @return mixed|string
     * @throws \Exception
     */
    protected function sendLeadsRequest($leadIds, $operation)
    {
        $params = [];
        foreach ($leadIds as $leadId) {
            $customer = $leadId['mObj'];
            if (intval($this->getCustomerOrderCollection($customer)->getSize()) > 0) {
                continue;
            }
            $info = $this->_data->getCustomer($customer, $this->_type);
            $info += [
                'FirstName' => $customer->getFirstname(),
                'LastName'  => $customer->getLastname(),
                'Email'     => $customer->getEmail(),
                'Company'   => 'N/A',
            ];
            if (isset($leadId['sid'])) {
                $info += ['Id' => $leadId['sid']];
            }
            $params[] = $info;
        }
        $response = $this->_job->sendBatchRequest($operation, $this->_type, $this->_json->serialize($params));
        $this->saveReports($operation, $this->_type, $response, $leadIds);
        return $response;
    }

    /**
     * @param Customer $customer
     * @return \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    protected function getCustomerOrderCollection($customer)
    {
        return $this->_helperData->getCustomerOrderCollection($customer->getId());
    }
}
